@@ -41,12 +41,13 @@ def count_reads_in_features(sam_filenames, gff_filename,
             r = (r,)
         for read in r:
             if read is not None:
-                samoutfile.write(read.original_sam_line.rstrip() +
-                                 "\tXF:Z:" + assignment + "\n")
+                read.optional_fields.append(('XF', assignment))
+                samoutfile.write(read.get_sam_line() + "\n")
 
-    if samouts != "":
+    if samouts != []:
         if len(samouts) != len(sam_filenames):
-            raise ValueError('Select the same number of SAM input and output files')
+            raise ValueError(
+                    'Select the same number of SAM input and output files')
         # Try to open samout files early in case any of them has issues
         for samout in samouts:
             with open(samout, 'w'):
@@ -73,12 +74,14 @@ def count_reads_in_features(sam_filenames, gff_filename,
                 try:
                     feature_id = f.attr[id_attribute]
                 except KeyError:
-                    raise ValueError("Feature %s does not contain a '%s' attribute" %
-                                     (f.name, id_attribute))
+                    raise ValueError(
+                            "Feature %s does not contain a '%s' attribute" %
+                            (f.name, id_attribute))
                 if stranded != "no" and f.iv.strand == ".":
-                    raise ValueError("Feature %s at %s does not have strand information but you are "
-                                     "running htseq-count in stranded mode. Use '--stranded=no'." %
-                                     (f.name, f.iv))
+                    raise ValueError(
+                            "Feature %s at %s does not have strand information but you are "
+                            "running htseq-count in stranded mode. Use '--stranded=no'." %
+                            (f.name, f.iv))
                 features[f.iv] += feature_id
                 counts[f.attr[id_attribute]] = 0
                 attributes[f.attr[id_attribute]] = [
@@ -116,22 +119,28 @@ def count_reads_in_features(sam_filenames, gff_filename,
     lowqual_all = []
     nonunique_all = []
     for isam, (sam_filename) in enumerate(sam_filenames):
-        if samouts != '':
+        if samouts != []:
             samoutfile = open(samouts[isam], 'w')
         else:
             samoutfile = None
 
         try:
-            if sam_filename != "-":
-                read_seq_file = SAM_or_BAM_Reader(sam_filename)
-                read_seq = read_seq_file
-                first_read = next(iter(read_seq))
-            else:
+            if sam_filename == "-":
                 read_seq_file = SAM_or_BAM_Reader(sys.stdin)
-                read_seq_iter = iter(read_seq_file)
+            else:
+                read_seq_file = SAM_or_BAM_Reader(sam_filename)
+            read_seq_iter = iter(read_seq_file)
+            # Catch empty BAM files
+            try:
                 first_read = next(read_seq_iter)
+                pe_mode = first_read.paired_end
+            except:
+                first_read = None
+                pe_mode = False
+            if first_read is not None:
                 read_seq = itertools.chain([first_read], read_seq_iter)
-            pe_mode = first_read.paired_end
+            else:
+                read_seq = []
         except:
             sys.stderr.write(
                 "Error occured when reading beginning of SAM/BAM file.\n")
@@ -175,7 +184,10 @@ def count_reads_in_features(sam_filenames, gff_filename,
                     try:
                         if r.optional_field("NH") > 1:
                             nonunique += 1
-                            write_to_samout(r, "__alignment_not_unique", samoutfile)
+                            write_to_samout(
+                                    r,
+                                    "__alignment_not_unique",
+                                    samoutfile)
                             if multimapped_mode == 'none':
                                 continue
                     except KeyError:
@@ -325,7 +337,8 @@ def count_reads_in_features(sam_filenames, gff_filename,
     print('\t'.join(["__alignment_not_unique"] + pad + [str(c) for c in nonunique_all]))
 
 
-def my_showwarning(message, category, filename, lineno=None, file=None, line=None):
+def my_showwarning(message, category, filename, lineno=None, file=None,
+                   line=None):
     sys.stderr.write("Warning: %s\n" % message)
 
 
@@ -396,9 +409,11 @@ def main():
             "suitable for Ensembl GTF files: gene_id)")
 
     pa.add_argument(
-            "--additional-attr", type=str, nargs='+',
-            default=(), help="Additional feature attributes (default: none, " +
-            "suitable for Ensembl GTF files: gene_name)")
+            "--additional-attr", type=str,
+            action='append',
+            default=[], help="Additional feature attributes (default: none, " +
+            "suitable for Ensembl GTF files: gene_name). Use multiple times " +
+            "for each different attribute")
 
     pa.add_argument(
             "-m", "--mode", dest="mode",
@@ -414,19 +429,20 @@ def main():
 
     pa.add_argument(
             "--secondary-alignments", dest="secondary_alignments", type=str,
-            choices=("score", "ignore"), default="score",
+            choices=("score", "ignore"), default="ignore",
             help="Whether to score secondary alignments (0x100 flag)")
 
     pa.add_argument(
             "--supplementary-alignments", dest="supplementary_alignments", type=str,
-            choices=("score", "ignore"), default="score",
+            choices=("score", "ignore"), default="ignore",
             help="Whether to score supplementary alignments (0x800 flag)")
 
     pa.add_argument(
-            "-o", "--samout", type=str, dest="samouts", nargs='+',
-            default="", help="write out all SAM alignment records into an output " +
-            "SAM file called SAMOUT, annotating each line with its feature assignment " +
-            "(as an optional field with tag 'XF')")
+            "-o", "--samout", type=str, dest="samouts",
+            action='append',
+            default=[], help="write out all SAM alignment records into " +
+            "SAM files (one per input file needed), annotating each line " +
+            "with its feature assignment (as an optional field with tag 'XF')")
 
     pa.add_argument(
             "-q", "--quiet", action="store_true", dest="quiet",
