@@ -9,21 +9,10 @@ import os
 import shlex
 import sys
 
-try:
-    import HTSeq
-    from HTSeq._HTSeq import *
-    #from _HTSeq import *
-except ImportError:
-    if os.path.isfile("setup.py"):
-        raise ImportError(
-            "Cannot import 'HTSeq' when working directory is HTSeq's own build directory.")
-    else:
-        raise
-
+import HTSeq
+from HTSeq._HTSeq import *
 from HTSeq._version import __version__
 
-#from vcf_reader import *
-
 #########################
 # Utils
 #########################
@@ -32,14 +21,14 @@ from HTSeq._version import __version__
 class FileOrSequence(object):
     """ The construcutor takes one argument, which may either be a string,
     which is interpreted as a file name (possibly with path), or a
-    connection, by which we mean a text file opened for reading, or 
-    any other object that can provide an iterator over strings 
+    connection, by which we mean a text file opened for reading, or
+    any other object that can provide an iterator over strings
     (lines of the file).
 
     The advantage of passing a file name instead of an already opened file
     is that if an iterator is requested several times, the file will be
     re-opened each time. If the file is already open, its lines can be read
-    only once, and then, the iterator stays exhausted.      
+    only once, and then, the iterator stays exhausted.
 
     Furthermore, if a file name is passed that end in ".gz" or ".gzip"
     (case insensitive), it is transparently gunzipped.
@@ -90,14 +79,13 @@ class FileOrSequence(object):
 #########################
 
 class GenomicFeature(object):
-
     """A genomic feature, i.e., an interval on a genome with metadata.
 
     At minimum, the following information should be provided by slots:
 
       name: a string identifying the feature (e.g., a gene symbol)
       type: a string giving the feature type (e.g., "gene", "exon")
-      iv: a GenomicInterval object specifying the feature locus     
+      iv: a GenomicInterval object specifying the feature locus
     """
 
     def __init__(self, name, type_, interval):
@@ -145,8 +133,8 @@ class GenomicFeature(object):
         attr_str = '; '.join(
             ['%s%s\"%s\"' % (ak, sep, attr[ak]) for ak in attr])
         return "\t".join(str(a) for a in (self.iv.chrom, source,
-                                          self.type, self.iv.start + 1, self.iv.end, score,
-                                          self.iv.strand, frame, attr_str)) + "\n"
+                         self.type, self.iv.start + 1, self.iv.end, score,
+                         self.iv.strand, frame, attr_str)) + "\n"
 
 
 _re_attr_main = re.compile("\s*([^\s\=]+)[\s=]+(.*)")
@@ -157,13 +145,16 @@ def parse_GFF_attribute_string(attrStr, extra_return_first_value=False):
     """Parses a GFF attribute string and returns it as a dictionary.
 
     If 'extra_return_first_value' is set, a pair is returned: the dictionary
-    and the value of the first attribute. This might be useful if this is the ID.
+    and the value of the first attribute. This might be useful if this is the
+    ID.
     """
     if attrStr.endswith("\n"):
         attrStr = attrStr[:-1]
     d = {}
     first_val = "_unnamed_"
-    for (i, attr) in zip(itertools.count(), _HTSeq.quotesafe_split(attrStr.encode())):
+    for (i, attr) in zip(
+            itertools.count(),
+            _HTSeq.quotesafe_split(attrStr.encode())):
         attr = attr.decode()
         if _re_attr_empty.match(attr):
             continue
@@ -176,7 +167,6 @@ def parse_GFF_attribute_string(attrStr, extra_return_first_value=False):
         val = mo.group(2)
         if val.startswith('"') and val.endswith('"'):
             val = val[1:-1]
-        #val = urllib.unquote( val )
         d[sys.intern(mo.group(1))] = sys.intern(val)
         if extra_return_first_value and i == 0:
             first_val = val
@@ -185,14 +175,14 @@ def parse_GFF_attribute_string(attrStr, extra_return_first_value=False):
     else:
         return d
 
+
 _re_gff_meta_comment = re.compile("##\s*(\S+)\s+(\S*)")
 
 
 class GFF_Reader(FileOrSequence):
-
     """Parse a GFF file
 
-    Pass the constructor either a file name or an iterator of lines of a 
+    Pass the constructor either a file name or an iterator of lines of a
     GFF files. If a file name is specified, it may refer to a gzip compressed
     file.
 
@@ -208,7 +198,6 @@ class GFF_Reader(FileOrSequence):
         for line in FileOrSequence.__iter__(self):
             if isinstance(line, bytes):
                 line = line.decode()
-
             if line == "\n":
                 continue
             if line.startswith('#'):
@@ -221,10 +210,15 @@ class GFF_Reader(FileOrSequence):
              strand, frame, attributeStr) = line.split("\t", 8)
             (attr, name) = parse_GFF_attribute_string(attributeStr, True)
             if self.end_included:
-                iv = GenomicInterval(seqname, int(start) - 1, int(end), strand)
+                iv = GenomicInterval(
+                        seqname,
+                        int(start) - 1, int(end),
+                        strand)
             else:
-                iv = GenomicInterval(seqname, int(
-                    start) - 1, int(end) - 1, strand)
+                iv = GenomicInterval(
+                        seqname,
+                        int(start) - 1, int(end) - 1,
+                        strand)
             f = GenomicFeature(name, feature, iv)
             if score != ".":
                 score = float(score)
@@ -238,7 +232,7 @@ class GFF_Reader(FileOrSequence):
 
 
 def make_feature_dict(feature_sequence):
-    """A feature dict is a convenient way to organize a sequence of Feature 
+    """A feature dict is a convenient way to organize a sequence of Feature
     object (which you have got, e.g., from parse_GFF).
 
     The function returns a dict with all the feature types as keys. Each value
@@ -248,8 +242,8 @@ def make_feature_dict(feature_sequence):
     An example makes this clear. Let's say you load the C. elegans GTF file
     from Ensemble and make a feature dict:
 
-    >>> worm_features_dict = HTSeq.make_feature_dict( HTSeq.parse_GFF( 
-    ...     "test_data/Caenorhabditis_elegans.WS200.55.gtf.gz" ) )
+    >>> worm_features_dict = HTSeq.make_feature_dict(HTSeq.parse_GFF(
+    ...     "test_data/Caenorhabditis_elegans.WS200.55.gtf.gz"))
 
     (This command may take a few minutes to deal with the 430,000 features
     in the GTF file. Note that you may need a lot of RAM if you have millions
@@ -277,8 +271,9 @@ def make_feature_dict(feature_sequence):
 #########################
 
 def read_chrom_lens(filename, delimiter="\t"):
-    return dict(((chrom, int(len))
-                 for chrom, len in csv.reader(open(filename), delimiter=delimiter)))
+    return dict(
+        ((chrom, int(len))
+         for chrom, len in csv.reader(open(filename), delimiter=delimiter)))
 
 
 #########################
@@ -300,6 +295,8 @@ class FastaReader(FileOrSequence):
 
     def __iter__(self):
         seq = None
+        name = None
+        descr = None
         for line in FileOrSequence.__iter__(self):
             if line.startswith(">"):
                 if seq:
@@ -326,6 +323,7 @@ class FastaReader(FileOrSequence):
 
     def get_sequence_lengths(self):
         seqname = None
+        length = 0
         seqlengths = {}
         for line in FileOrSequence.__iter__(self):
             if line.startswith(">"):
@@ -354,12 +352,13 @@ class FastaReader(FileOrSequence):
     def build_index(self, force=False):
         self._import_pysam()
         if not isinstance(self.fos, str):
-            raise TypeError("This function only works with FastaReader objects " +
-                            "connected to a fasta file via file name")
+            raise TypeError(
+                "This function only works with FastaReader objects " +
+                "connected to a fasta file via file name")
         index_filename = self.fos + ".fai"
         if os.access(index_filename, os.R_OK):
-            if (not force) and os.stat( self.filename_or_sequence ).st_mtime <= \
-                    os.stat(index_filename).st_mtime:
+            if (not force) and os.stat(self.filename_or_sequence).st_mtime <= \
+                 os.stat(index_filename).st_mtime:
                 # index is up to date
                 return
         pysam.faidx(self.fos)
@@ -371,11 +370,13 @@ class FastaReader(FileOrSequence):
         if not isinstance(iv, GenomicInterval):
             raise TypeError("GenomicInterval expected as key.")
         if not isinstance(self.fos, str):
-            raise TypeError("This function only works with FastaReader objects " +
-                            "connected to a fasta file via file name")
+            raise TypeError(
+                "This function only works with FastaReader objects " +
+                "connected to a fasta file via file name")
         self._import_pysam()
-        fasta = pysam.faidx(self.fos, "%s:%d-%d" %
-                            (iv.chrom, iv.start, iv.end - 1))
+        fasta = pysam.faidx(
+                self.fos,
+                "%s:%d-%d" % (iv.chrom, iv.start, iv.end - 1))
         ans = list(FastaReader(fasta))
         assert len(ans) == 1
         ans[0].name = str(iv)
@@ -408,28 +409,35 @@ class FastqReader(FileOrSequence):
             qual = next(fin)
             if qual == "":
                 if id1 != "":
-                    warnings.warn("Number of lines in FASTQ file is not "
-                                  "a multiple of 4. Discarding the last, "
-                                  "incomplete record")
+                    warnings.warn(
+                        "Number of lines in FASTQ file is not "
+                        "a multiple of 4. Discarding the last, "
+                        "incomplete record")
                 break
 
             if not qual.endswith("\n"):
                 qual += "\n"
             if not id1.startswith("@"):
-                raise ValueError("Primary ID line in FASTQ file does"
-                                 "not start with '@'. Either this is not FASTQ data or the parser got out of sync.")
+                raise ValueError(
+                    "Primary ID line in FASTQ file does "
+                    "not start with '@'. Either this is not FASTQ data or the "
+                    "parser got out of sync.")
             if not id2.startswith("+"):
-                raise ValueError("Secondary ID line in FASTQ file does"
-                                 "not start with '+'. Maybe got out of sync.")
+                raise ValueError(
+                    "Secondary ID line in FASTQ file does"
+                    "not start with '+'. Maybe got out of sync.")
             if len(id2) > 2 and id1[1:] != id2[1:]:
-                raise ValueError("Primary and secondary ID line in FASTQ"
-                                 "disagree.")
+                raise ValueError(
+                    "Primary and secondary ID line in FASTQ"
+                    "disagree.")
 
             if self.raw_iterator:
                 s = (seq[:-1], id1[1:-1], qual[:-1], self.qual_scale)
             else:
-                s = SequenceWithQualities(seq[:-1].encode(), id1[1:-1],
-                                          qual[:-1].encode(), self.qual_scale)
+                s = SequenceWithQualities(
+                        seq[:-1].encode(), id1[1:-1],
+                        qual[:-1].encode(),
+                        self.qual_scale)
             yield s
 
 
@@ -445,8 +453,10 @@ class BowtieReader(FileOrSequence):
             except ValueError:
                 if line.startswith("Reported "):
                     continue
-                warnings.warn("BowtieReader: Ignoring the following line, which could not be parsed:\n%s\n" % line,
-                              RuntimeWarning)
+                warnings.warn(
+                    "BowtieReader: Ignoring the following line, which could "
+                    "not be parsed:\n%s\n" % line,
+                    RuntimeWarning)
             yield algnt
 
 
@@ -472,14 +482,14 @@ def bundle_multiple_alignments(sequence_of_alignments):
 class SolexaExportAlignment(Alignment):
     """Iterating over SolexaExportReader objects will yield SoelxaExportRecord
     objects. These have four fields:
-       read          - a SequenceWithQualities object 
+       read          - a SequenceWithQualities object
        aligned       - a boolean, indicating whether the object was aligned
        iv            - a GenomicInterval giving the alignment (or None, if not aligned)
        passed_filter - a boolean, indicating whether the object passed the filter
        nomatch_code  - a code indicating why no match was found (or None, if the
          read was aligned)
 
-    As long as 'aligned' is True, a SolexaExportRecord can be treated as an 
+    As long as 'aligned' is True, a SolexaExportRecord can be treated as an
     Alignment object.
     """
 
@@ -490,10 +500,10 @@ class SolexaExportAlignment(Alignment):
     def __repr__(self):
         if self.aligned:
             return "< %s object: Read '%s', aligned to %s >" % (
-                self.__class__.__name__, self.read.name, self.iv)
+              self.__class__.__name__, self.read.name, self.iv)
         else:
             return "< %s object: Non-aligned read '%s' >" % (
-                self.__class__.__name__, self.read.name)
+              self.__class__.__name__, self.read.name)
 
 
 class SolexaExportReader(FileOrSequence):
@@ -514,15 +524,28 @@ class SolexaExportReader(FileOrSequence):
         if line[-1] == "\n":
             line = line[:-1]
         res = {}
-        (res['machine'], res['run_number'], res['lane'], res['tile'], res['x_coord'],
-         res['y_coord'], res['index_string'], res['read_nbr'], res['read_seq'],
-         res['qual_str'], res['chrom'], res[
-             'contig'], res['pos'], res['strand'],
-         res['match_descr'], res['single_read_algnt_score'],
-         res['paired_read_algnt_score'], res[
-             'partner_chrom'], res['partner_contig'],
-         res['partner_offset'], res['partner_strand'], res['passed_filtering'] ) \
-            = line.split("\t")
+        (res['machine'],
+         res['run_number'],
+         res['lane'],
+         res['tile'],
+         res['x_coord'],
+         res['y_coord'],
+         res['index_string'],
+         res['read_nbr'],
+         res['read_seq'],
+         res['qual_str'],
+         res['chrom'],
+         res['contig'],
+         res['pos'],
+         res['strand'],
+         res['match_descr'],
+         res['single_read_algnt_score'],
+         res['paired_read_algnt_score'],
+         res['partner_chrom'],
+         res['partner_contig'],
+         res['partner_offset'],
+         res['partner_strand'],
+         res['passed_filtering']) = line.split("\t")
         return res
 
     def __iter__(self):
@@ -530,20 +553,25 @@ class SolexaExportReader(FileOrSequence):
             record = SolexaExportAlignment()
             fields = SolexaExportReader.parse_line_bare(line)
             if fields['read_nbr'] != "1":
-                warnings.warn("Paired-end read encountered. PE is so far supported only for " +
-                              "SAM files, not yet for SolexaExport. All PE-related fields are ignored. ")
+                warnings.warn(
+                    "Paired-end read encountered. PE is so far supported only "
+                    "for SAM files, not yet for SolexaExport. All PE-related "
+                    "fields are ignored.")
             record.read = SequenceWithQualities(
                 fields['read_seq'],
-                "%s:%s:%s:%s:%s#0" % (fields['machine'], fields['lane'], fields['tile'],
-                                      fields['x_coord'], fields['y_coord']),
+                "%s:%s:%s:%s:%s#0" % (fields['machine'],
+                                      fields['lane'],
+                                      fields['tile'],
+                                      fields['x_coord'],
+                                      fields['y_coord']),
                 fields['qual_str'], self.qualscale)
             if fields['passed_filtering'] == 'Y':
                 record.passed_filter = True
             elif fields['passed_filtering'] == 'N':
                 record.passed_filter = False
             else:
-                raise ValueError("Illegal 'passed filter' value in Solexa export data: '%s'." % fields[
-                                 'passed_filtering'])
+                raise ValueError(
+                    "Illegal 'passed filter' value in Solexa export data: '%s'." % fields['passed_filtering'])
             record.index_string = fields['index_string']
             if fields['pos'] == '':
                 record.iv = None
@@ -560,13 +588,14 @@ class SolexaExportReader(FileOrSequence):
                 chrom = fields['chrom']
                 if fields['chrom'] == "":
                     chrom = fields['contig']
-                record.iv = GenomicInterval(chrom, start,
-                                            start + len(fields['read_seq']), strand)
+                record.iv = GenomicInterval(
+                    chrom, start,
+                    start + len(fields['read_seq']), strand)
             yield record
 
 
 class SAM_Reader(FileOrSequence):
-    """A SAM_Reader object is associated with a SAM file that 
+    """A SAM_Reader object is associated with a SAM file that
     contains short read alignments. It can generate an iterator of Alignment
     objects."""
 
@@ -584,7 +613,6 @@ class SAM_Reader(FileOrSequence):
 
 
 class GenomicArrayOfSets(GenomicArray):
-
     """A GenomicArrayOfSets is a specialization of GenomicArray that allows to store
     sets of objects. On construction, the step vectors are initialized with empty sets.
     By using the 'add_value' method, objects can be added to intervals. If an object
@@ -606,11 +634,50 @@ class GenomicArrayOfSets(GenomicArray):
 # paired-end handling
 ###########################
 
-def pair_SAM_alignments(alignments, bundle=False):
+def pair_SAM_alignments(
+        alignments,
+        bundle=False,
+        primary_only=False):
+    '''Iterate over SAM aligments, name-sorted paired-end
+
+    Args:
+        alignments (iterator of SAM/BAM alignments): the alignments to wrap
+        bundle (bool): if True, bundle all alignments from one read pair into a
+            single yield. If False (default), each pair of alignments is
+            yielded separately.
+        primary_only (bool): for each read, consider only the primary line
+            (SAM flag 0x900 = 0). The SAM specification requires one and only
+            one of those for each read.
+
+    Yields:
+        2-tuples with each pair of alignments or, if bundle==True, each bundled
+        list of alignments.
+    '''
 
     mate_missing_count = [0]
 
     def process_list(almnt_list):
+        '''Transform a list of alignment with the same read name into pairs
+
+        Args:
+            almnt_list (list): alignments to process
+
+        Yields:
+            each pair of alignments.
+
+        This function is needed because each line of a BAM file is not a read
+        but an alignment. For uniquely mapped and unmapped reads, those two are
+        the same. For multimapped reads, however, there can be more than one
+        alignment for each read. Also, it is normal for a mapper to uniquely
+        map one read and multimap its mate.
+
+        This function goes down the list of alignments for a given read name
+        and tries to find the first mate. So if read 1 is uniquely mapped but
+        read 2 is mapped 4 times, only (read 1, read 2 - first occurrence) will
+        yield; the other 3 alignments of read 2 are ignored.
+        '''
+
+
         while len(almnt_list) > 0:
             a1 = almnt_list.pop(0)
             # Find its mate
@@ -622,14 +689,15 @@ def pair_SAM_alignments(alignments, bundle=False):
                 if not (a1.aligned and a2.aligned):
                     break
                 if a1.iv.chrom == a2.mate_start.chrom and a1.iv.start == a2.mate_start.pos and \
-                        a2.iv.chrom == a1.mate_start.chrom and a2.iv.start == a1.mate_start.pos:
+                   a2.iv.chrom == a1.mate_start.chrom and a2.iv.start == a1.mate_start.pos:
                     break
             else:
                 if a1.mate_aligned:
                     mate_missing_count[0] += 1
                     if mate_missing_count[0] == 1:
-                        warnings.warn("Read " + a1.read.name + " claims to have an aligned mate " +
-                                      "which could not be found in an adjacent line.")
+                        warnings.warn(
+                            "Read " + a1.read.name + " claims to have an aligned mate " +
+                            "which could not be found in an adjacent line.")
                 a2 = None
             if a2 is not None:
                 almnt_list.remove(a2)
@@ -648,6 +716,9 @@ def pair_SAM_alignments(alignments, bundle=False):
         if almnt.pe_which == "unknown":
             raise ValueError(
                 "Paired-end read found with 'unknown' 'pe_which' status.")
+        # FIXME: almnt.not_primary_alignment currently means secondary
+        if primary_only and (almnt.not_primary_alignment or almnt.supplementary):
+            continue
         if almnt.read.name == current_name:
             almnt_list.append(almnt)
         else:
@@ -668,18 +739,35 @@ def pair_SAM_alignments(alignments, bundle=False):
                       mate_missing_count[0])
 
 
-def pair_SAM_alignments_with_buffer(alignments, max_buffer_size=30000000):
+def pair_SAM_alignments_with_buffer(
+        alignments,
+        max_buffer_size=30000000,
+        primary_only=False):
+    '''Iterate over SAM aligments with buffer, position-sorted paired-end
+
+    Args:
+        alignments (iterator of SAM/BAM alignments): the alignments to wrap
+        max_buffer_size (int): maxmal numer of alignments to keep in memory.
+        primary_only (bool): for each read, consider only the primary line
+            (SAM flag 0x900 = 0). The SAM specification requires one and only
+            one of those for each read.
+
+    Yields:
+        2-tuples with each pair of alignments.
+    '''
 
     almnt_buffer = {}
     ambiguous_pairing_counter = 0
     for almnt in alignments:
-
         if not almnt.paired_end:
             raise ValueError(
                 "Sequence of paired-end alignments expected, but got single-end alignment.")
         if almnt.pe_which == "unknown":
             raise ValueError(
                 "Cannot process paired-end alignment found with 'unknown' 'pe_which' status.")
+        # FIXME: almnt.not_primary_alignment currently means secondary
+        if primary_only and (almnt.not_primary_alignment or almnt.supplementary):
+            continue
 
         matekey = (
             almnt.read.name,
@@ -720,8 +808,9 @@ def pair_SAM_alignments_with_buffer(alignments, max_buffer_size=30000000):
                     "Maximum alignment buffer size exceeded while pairing SAM alignments.")
 
     if len(almnt_buffer) > 0:
-        warnings.warn("Mate records missing for %d records; first such record: %s." %
-                      (len(almnt_buffer), str(list(almnt_buffer.values())[0][0])))
+        warnings.warn(
+            "Mate records missing for %d records; first such record: %s." %
+            (len(almnt_buffer), str(list(almnt_buffer.values())[0][0])))
         for almnt_list in list(almnt_buffer.values()):
             for almnt in almnt_list:
                 if almnt.pe_which == "first":
@@ -730,8 +819,9 @@ def pair_SAM_alignments_with_buffer(alignments, max_buffer_size=30000000):
                     yield (None, almnt)
 
     if ambiguous_pairing_counter > 0:
-        warnings.warn("Mate pairing was ambiguous for %d records; mate key for first such record: %s." %
-                      (ambiguous_pairing_counter, str(ambiguous_pairing_first_occurance)))
+        warnings.warn(
+            "Mate pairing was ambiguous for %d records; mate key for first such record: %s." %
+            (ambiguous_pairing_counter, str(ambiguous_pairing_first_occurance)))
 
 
 ###########################
@@ -756,7 +846,8 @@ _vcf_typemap = {
 
 class VariantCall(object):
 
-    def __init__(self, chrom=None, pos=None, identifier=None, ref=None, alt=None, qual=None, filtr=None, info=None):
+    def __init__(self, chrom=None, pos=None, identifier=None, ref=None,
+                 alt=None, qual=None, filtr=None, info=None):
         self.chrom = chrom
         self.pos = pos
         self.id = identifier
@@ -785,12 +876,10 @@ class VariantCall(object):
         ret = cls()
         if nsamples == 0:
             ret.format = None
-            ret.chrom, ret.pos, ret.id, ret.ref, ret.alt, ret.qual, ret.filter, ret.info = line.rstrip(
-                "\n").split("\t", 7)
+            ret.chrom, ret.pos, ret.id, ret.ref, ret.alt, ret.qual, ret.filter, ret.info = line.rstrip("\n").split("\t", 7)
         else:
             lsplit = line.rstrip("\n").split("\t")
-            ret.chrom, ret.pos, ret.id, ret.ref, ret.alt, ret.qual, ret.filter, ret.info = lsplit[
-                :8]
+            ret.chrom, ret.pos, ret.id, ret.ref, ret.alt, ret.qual, ret.filter, ret.info = lsplit[:8]
             ret.format = lsplit[8].split(":")
             ret.samples = {}
             spos = 9
@@ -874,10 +963,11 @@ class VCF_Reader(FileOrSequence):
 
     def make_info_dict(self):
         self.infodict = dict(
-            (key, _vcf_typemap[self.info[key]["Type"]]) for key in list(self.info.keys()))
+            (key,
+             _vcf_typemap[self.info[key]["Type"]]) for key in list(self.info.keys()))
 
     def parse_meta(self, header_filename=None):
-        if header_filename == None:
+        if header_filename is None:
             the_iter = FileOrSequence.__iter__(self)
         else:
             the_iter = open(header_filename, "r")
@@ -917,7 +1007,7 @@ class VCF_Reader(FileOrSequence):
 
     def meta_info(self, header_filename=None):
         ret = []
-        if header_filename == None:
+        if header_filename is None:
             the_iter = FileOrSequence.__iter__(self)
         else:
             the_iter = open(header_filename, "r")
@@ -1010,7 +1100,6 @@ class BAM_Reader(object):
         sf = pysam.Samfile(self.filename, "rb", check_sq=self.check_sq)
         self.record_no = 0
         for pa in sf:
-            # yield SAM_Alignment.from_pysam_AlignedRead( pa, sf )
             yield SAM_Alignment.from_pysam_AlignedSegment(pa, sf)
             self.record_no += 1
 
@@ -1056,8 +1145,8 @@ class BAM_Reader(object):
 
 
 class BAM_Writer(object):
-
-    def __init__(self, filename, template=None, referencenames=None, referencelengths=None, text=None, header=None):
+    def __init__(self, filename, template=None, referencenames=None,
+                 referencelengths=None, text=None, header=None):
         try:
             import pysam
         except ImportError:
@@ -1071,15 +1160,20 @@ class BAM_Writer(object):
         self.referencelengths = referencelengths
         self.text = text
         self.header = header
-        self.sf = pysam.Samfile(self.filename, mode="wb", template=self.template, referencenames=self.referencenames,
-                                referencelengths=self.referencelengths, text=self.text, header=self.header)
+        self.sf = pysam.Samfile(
+                self.filename,
+                mode="wb",
+                template=self.template,
+                referencenames=self.referencenames,
+                referencelengths=self.referencelengths,
+                text=self.text,
+                header=self.header)
 
     @classmethod
     def from_BAM_Reader(cls, fn, br):
         return BAM_Writer(filename=fn, header=br.get_header_dict())
 
     def write(self, alnmt):
-        #self.sf.write( alnmt.to_pysam_AlignedRead( self.sf ) )
         self.sf.write(alnmt.to_pysam_AlignedSegment(self.sf))
 
     def close(self):
@@ -1100,13 +1194,21 @@ class BED_Reader(FileOrSequence):
                 raise ValueError("BED file line contains less than 3 fields")
             if len(fields) > 9:
                 raise ValueError("BED file line contains more than 9 fields")
-            iv = GenomicInterval(fields[0], int(fields[1]), int(
-                fields[2]), fields[5] if len(fields) > 5 else ".")
-            f = GenomicFeature(fields[3] if len(
-                fields) > 3 else "unnamed", "BED line", iv)
+            iv = GenomicInterval(
+                fields[0],
+                int(fields[1]),
+                int(fields[2]),
+                fields[5] if len(fields) > 5 else ".")
+            f = GenomicFeature(
+                fields[3] if len(fields) > 3 else "unnamed",
+                "BED line",
+                iv)
             f.score = float(fields[4]) if len(fields) > 4 else None
-            f.thick = GenomicInterval(iv.chrom, int(fields[6]), int(
-                fields[7]), iv.strand) if len(fields) > 7 else None
+            f.thick = GenomicInterval(
+                iv.chrom,
+                int(fields[6]),
+                int(fields[7]),
+                iv.strand) if len(fields) > 7 else None
             f.itemRgb = [int(a) for a in fields[8].split(",")
                          ] if len(fields) > 8 else None
             yield(f)
